#!/usr/bin/python
# Authors:
#     Endi S. Dewata <edewata@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Copyright (C) 2015 Red Hat, Inc.
# All rights reserved.
#

import base64
import getopt
import ldap
import sys


class RestoreSubsystemUser(object):

    def __init__(self):
        self.description = 'Restore subsystem user'
        self.subsystem = None
        self.verbose = False
        self.silent = False

    def execute(self):

        state = self.subsystem.config['cs.state']
        if state != '1':
            if not self.silent:
                print 'Subsystem not yet configured'
            return

        type = self.subsystem.config['cs.type']
        hostname = self.subsystem.config['machineName']
        port = self.subsystem.config['service.securePort']

        user_id = '%s-%s-%s' % (type, hostname, port)

        cert = self.subsystem.get_system_cert('subsystem')
        version = cert['version']
        serial_number = cert['serial_number']
        issuer = cert['issuer']
        subject = cert['subject']

        description = '%s;%s;%s;%s' % (version, serial_number, issuer, subject)
        if self.verbose:
            print 'Subsystem certificate: %s' % description

        certificate = cert['certificate']
        if self.verbose:
            print '-----BEGIN CERTIFICATE-----'
            print base64.b64encode(certificate)
            print '-----END CERTIFICATE-----'

        users = self.subsystem.find_system_users(certificate=certificate)
        members = self.subsystem.get_system_group_members('Subsystem Group')

        for user in users:
            if self.verbose:
                print 'User %s has subsystem certificate' % user['uid']

            if user['uid'] in members:
                if self.verbose:
                    print 'User already in Subsystem Group'

                if 'description' in user:
                    desc = user['description']
                else:
                    desc = None

                if desc == description:
                    if self.verbose:
                        print 'User has the correct certificate mapping'

                    if not self.silent:
                        print 'Subsystem user %s is OK' % user['uid']

                else:
                    changes = {}
                    changes['description'] = description
                    self.subsystem.modify_system_user(user_id, changes)

                    if self.verbose:
                        print 'Certificate mapping fixed'

                    if not self.silent:
                        print 'Subsystem user %s restored' % user['uid']

                return

        # If the user with the subsystem certificate doesn't exist,
        # or if the user exists but not in the Subsystem Group,
        # create a new subsystem user

        try:
            user = {}
            user['uid'] = user_id
            user['cn'] = user_id
            user['sn'] = user_id
            user['type'] = 'agentType'
            user['description'] = description
            user['certificate'] = certificate
            user['state'] = '1'

            self.subsystem.add_system_user(user)
            if self.verbose:
                print 'New subsystem user %s added' % user_id

        except ldap.ALREADY_EXISTS:
            if self.verbose:
                print 'Subsystem user %s already exists' % user_id
            user = self.subsystem.get_system_user(user_id)

            changes = {}
            changes['description'] = description
            changes['certificate'] = certificate
            self.subsystem.modify_system_user(user_id, changes)
            if self.verbose:
                print 'User certificate updated'

        try:
            self.subsystem.add_system_group_member('Subsystem Group', user_id)
            if self.verbose:
                print 'User added to Subsystem Group'

        except ldap.TYPE_OR_VALUE_EXISTS:
            if self.verbose:
                print 'User already in Subsystem Group'

        if not self.silent:
            print 'Subsystem user %s restored' % user_id
